using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.ServiceModel;
using System.Runtime.Serialization;

namespace ClockMark
{
    [Serializable]
    public class Settings
    {
        private bool autoStartLogin;
        private bool autoStartWindows;
        private bool desktopIcon;
        private bool quickLaunchIcon;
        private bool showInSystemTray;
        private bool systemTrayOnly;
        private bool clickThrough;
        private bool unmovableWindow;
        private bool alwaysOnTop;

        private string username;
        private string hashedPassword;
        private string encryptedPassword;
        private string server;

        private bool autoSync;
        private bool syncOnShutdown;
        private bool askFirst;
        private DateTime lastSync;

        [DataMember]
        public bool AutoStartLogin
        {
            get { return autoStartLogin; }
            set { autoStartLogin = value; }
        }

        [DataMember]
        public bool AutoStartWindows
        {
            get { return autoStartWindows; }
            set { autoStartWindows = value; }
        }

        [DataMember]
        public bool DesktopIcon
        {
            get { return desktopIcon; }
            set { desktopIcon = value; }
        }

        [DataMember]
        public bool QuickLaunchIcon
        {
            get { return quickLaunchIcon; }
            set { quickLaunchIcon = value; }
        }

        [DataMember]
        public bool ShowInSystemTray
        {
            get { return showInSystemTray; }
            set { showInSystemTray = value; }
        }

        [DataMember]
        public bool SystemTrayOnly
        {
            get { return systemTrayOnly; }
            set { systemTrayOnly = value; }
        }

        [DataMember]
        public bool ClickThrough
        {
            get { return clickThrough; }
            set { clickThrough = value; }
        }

        [DataMember]
        public bool UnmovableWindow
        {
            get { return unmovableWindow; }
            set { unmovableWindow = value; }
        }

        [DataMember]
        public bool AlwaysOnTop
        {
            get { return alwaysOnTop; }
            set { alwaysOnTop = value; }
        }

        [DataMember]
        public string Username
        {
            get { return username; }
            set { username = value; }
        }

        [DataMember]
        public string HashedPassword
        {
            get { return hashedPassword; }
            set { hashedPassword = value; }
        }

        [DataMember]
        public string EncryptedPassword
        {
            get { return encryptedPassword; }
            set { encryptedPassword = value; }
        }

        [DataMember]
        public string Server
        {
            get { return server; }
            set { server = value; }
        }

        [DataMember]
        public bool AutoSync
        {
            get { return autoSync; }
            set { autoSync = value; }
        }

        [DataMember]
        public bool SyncOnShutdown
        {
            get { return syncOnShutdown; }
            set { syncOnShutdown = value; }
        }

        [DataMember]
        public bool AskFirst
        {
            get { return askFirst; }
            set { askFirst = value; }
        }

        [DataMember]
        public DateTime LastSync
        {
            get { return lastSync; }
            set { lastSync = value; }
        }

        public Settings()
        {
        }

        public bool FromString(string settings)
        {
            try
            {
                XmlDocument xd = new XmlDocument();
                xd.InnerXml = settings;

                this.AutoStartLogin = Boolean.Parse(xd.SelectSingleNode("Settings/AutoStartLogin").InnerText);
                this.AutoStartWindows = Boolean.Parse(xd.SelectSingleNode("Settings/AutoStartWindows").InnerText);
                this.DesktopIcon = Boolean.Parse(xd.SelectSingleNode("Settings/DesktopIcon").InnerText);
                this.QuickLaunchIcon = Boolean.Parse(xd.SelectSingleNode("Settings/QuickLaunchIcon").InnerText);
                this.ShowInSystemTray = Boolean.Parse(xd.SelectSingleNode("Settings/ShowInSystemTray").InnerText);
                this.ClickThrough = Boolean.Parse(xd.SelectSingleNode("Settings/ClickThrough").InnerText);
                this.DesktopIcon = Boolean.Parse(xd.SelectSingleNode("Settings/DesktopIcon").InnerText);
                this.UnmovableWindow = Boolean.Parse(xd.SelectSingleNode("Settings/UnmovableWindow").InnerText);
                this.AlwaysOnTop = Boolean.Parse(xd.SelectSingleNode("Settings/AlwaysOnTop").InnerText);
                this.Username = xd.SelectSingleNode("Settings/Username").InnerText;
                this.HashedPassword = xd.SelectSingleNode("Settings/Password").InnerText;
                this.Server = xd.SelectSingleNode("Settings/Server").InnerText;
                this.AutoSync = Boolean.Parse(xd.SelectSingleNode("Settings/AutoSync").InnerText);
                this.SyncOnShutdown = Boolean.Parse(xd.SelectSingleNode("Settings/SyncOnShutdown").InnerText);
                this.AskFirst = Boolean.Parse(xd.SelectSingleNode("Settings/AskFirst").InnerText);
                this.LastSync = DateTime.Parse(xd.SelectSingleNode("Settings/LastSync").InnerText);

                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("Could not convert xml string to settings.");
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return false;
            }
        }

        public override string ToString()
        {
            try
            {
                XmlDocument xd = new XmlDocument();

                XmlDeclaration xdec = xd.CreateXmlDeclaration("1.0", "utf-8", null);
                xd.AppendChild(xdec);

                XmlElement xroot = xd.CreateElement("Settings");
                xd.AppendChild(xroot);

                XmlElement xe = xd.CreateElement("AutoStartLogin");
                XmlText xt = xd.CreateTextNode(this.AutoStartLogin.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("AutoStartWindows");
                xt = xd.CreateTextNode(this.AutoStartWindows.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("DesktopIcon");
                xt = xd.CreateTextNode(this.AutoStartWindows.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("QuickLaunchIcon");
                xt = xd.CreateTextNode(this.DesktopIcon.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("ShowInSystemTray");
                xt = xd.CreateTextNode(this.ShowInSystemTray.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("ClickThrough");
                xt = xd.CreateTextNode(this.ClickThrough.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("DesktopIcon");
                xt = xd.CreateTextNode(this.DesktopIcon.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("UnmovableWindow");
                xt = xd.CreateTextNode(this.UnmovableWindow.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("AlwaysOnTop");
                xt = xd.CreateTextNode(this.AlwaysOnTop.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("Username");
                xt = xd.CreateTextNode(this.Username);
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("Password");
                xt = xd.CreateTextNode(this.HashedPassword);
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("Server");
                xt = xd.CreateTextNode(this.Server);
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("AutoSync");
                xt = xd.CreateTextNode(this.AutoSync.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("SyncOnShutdown");
                xt = xd.CreateTextNode(this.SyncOnShutdown.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("AskFirst");
                xt = xd.CreateTextNode(this.AskFirst.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                xe = xd.CreateElement("LastSync");
                xt = xd.CreateTextNode(this.LastSync.ToString());
                xe.AppendChild(xt);
                xroot.AppendChild(xe);

                return xd.InnerXml.ToString();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("Could not convert settings to xml string.");
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return null;
            }
        }
    }
}
