using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Text;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Net;
using System.Net.Sockets;
using System.Net.NetworkInformation;
using System.IO;
using System.Text.RegularExpressions;
using System.Security.Cryptography;

namespace ClockMark
{
    // View part of the Model/View/Controller pattern
	public partial class frmClockMark : Form
	{
		#region Variables

        // local variables

        private bool ClockMovementAllowed;
		private bool AlarmsExpanded;

		// Defines the ticking style used for the second and minute hands of the analog clock. 
		public enum TickStyle
		{
			// Smooth ticking style. For example if used with second hand it will be updated every millisecond.
			Smooth,
			// Normal ticking style. For example if used with second hand it will be updated every second only.
			Normal
		}

		private bool drawCentralPoint = true;
		private bool drawNumerals = true;
		private bool drawRim = true;
		private bool drawDropShadow = true;
		private bool drawSecondHandShadow = true;
		private bool drawMinuteHandShadow = true;
		private bool drawHourHandShadow = true;
		private bool drawSecondHand = true;
		private bool drawMinuteHand = true;
		private bool drawHourHand = true;
		private bool drawDate = true;
		private bool drawAmblem = true;

		private Color dropShadowColor = Color.Black;

		private Color secondHandDropShadowColor = Color.Gray;
		private Color hourHandDropShadowColor = Color.Gray;
		private Color minuteHandDropShadowColor = Color.Gray;

		private Color faceColor1 = Color.RoyalBlue;
		private Color faceColor2 = Color.SkyBlue;
		private Color rimColor1 = Color.RoyalBlue;
		private Color rimColor2 = Color.SkyBlue;
		private Color numeralColor = Color.WhiteSmoke;
		private Color secondHandColor = Color.Tomato;

		private LinearGradientBrush linGradBrush;
		private LineCap secLineEndCap = LineCap.Round;
		//private Point dropShadowOffset = new Point(5,5);
		private LinearGradientMode faceGradientMode = LinearGradientMode.ForwardDiagonal;
		private LinearGradientMode rimGradientMode = LinearGradientMode.BackwardDiagonal;

		private Color hourHandColor = Color.Gainsboro;
		private Color minHandColor = Color.WhiteSmoke;

		private float radius;
		private float midx;
		private float midy;
		private float y;
		private float x;
		private float fontsize;
		private Font textFont;
		private int min;
		private int hour;
		private double sec;
		private Image imgClock;

		float minuteAngle;
		double secondAngle;
		double hourAngle;

		private TickStyle secondHandTickStyle = TickStyle.Normal;
		private TickStyle minHandTickStyle = TickStyle.Normal;

		private Point dropShadowOffset;

        // global variables and properties

        private DateTime dateTimeClock;
        private string[] uuid;
        private Settings clockMarkSettings;
        private Alarms clockMarkAlarms;

        public string[] UUID
        {
            get { return uuid; }
            set { uuid = value; }
        }
        
        public Settings ClockMarkSettings
        {
            get { return clockMarkSettings; }
            set { clockMarkSettings = value; }
        }
        
        public Alarms ClockMarkAlarms
        {
            get { return clockMarkAlarms; }
            set { clockMarkAlarms = value; }
        }

        private bool isUserLoggedIn;

        public bool IsUserLoggedIn
        {
            get { return isUserLoggedIn; }
            set { isUserLoggedIn = value; }
        }

        // encryption/decryption
        private byte[] key;
        private byte[] iv;

		#endregion

		#region Rounded corners

		[DllImport("Gdi32.dll", EntryPoint = "CreateRoundRectRgn")]
		private static extern IntPtr CreateRoundRectRgn
		(
			int nLeftRect, // x-coordinate of upper-left corner
			int nTopRect, // y-coordinate of upper-left corner
			int nRightRect, // x-coordinate of lower-right corner
			int nBottomRect, // y-coordinate of lower-right corner
			int nWidthEllipse, // height of ellipse
			int nHeightEllipse // width of ellipse
		);

		#endregion

		#region Move the form without titlebar

		public const int WM_NCLBUTTONDOWN = 0xA1;
		public const int HT_CAPTION = 0x2;

		[DllImportAttribute("user32.dll")]
		public static extern int SendMessage(IntPtr hWnd,
						 int Msg, int wParam, int lParam);
		[DllImportAttribute("user32.dll")]
		public static extern bool ReleaseCapture();

		#endregion

		#region Constructor

		public frmClockMark()
		{
			InitializeComponent();

            this.Icon = Properties.Resources.ClockMark;
            this.niClockMark.Icon = Properties.Resources.ClockMark;
			
			this.SetStyle(ControlStyles.DoubleBuffer, true);
			this.SetStyle(ControlStyles.UserPaint, true);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);

			this.ClockMovementAllowed = true;

			this.dateTimeClock = new System.DateTime(((long)(0)));

            this.ClockMarkSettings = this.LoadSettings();
            this.UUID = this.GetUUID();
            this.ClockMarkAlarms = this.LoadAlarms();
            
            // logging and synchronizing
            this.IsUserLoggedIn = false;

            if (this.ConnectToTheServer(this.UUID))
            {
                this.IsUserLoggedIn = true;

                if (this.SyncronizeWithServer(this.UUID))
                {
                }
            }

            #region Rounded corners

            this.Region = System.Drawing.Region.FromHrgn(
				CreateRoundRectRgn(0, 0, 
					this.Width, this.Height, 
					20, 20));

            this.lbAdd.Region = System.Drawing.Region.FromHrgn(
                CreateRoundRectRgn(0, 0,
                    this.lbAdd.Width, this.lbAdd.Height,
                    12, 12));
            this.lbDelete.Region = System.Drawing.Region.FromHrgn(
                CreateRoundRectRgn(0, 0,
                    this.lbDelete.Width, this.lbDelete.Height,
                    12, 12));
            this.lbList.Region = System.Drawing.Region.FromHrgn(
                CreateRoundRectRgn(0, 0,
                    this.lbList.Width, this.lbList.Height,
                    12, 12));
            this.lbView.Region = System.Drawing.Region.FromHrgn(
                CreateRoundRectRgn(0, 0,
                    this.lbView.Width, this.lbView.Height,
                    12, 12));
            this.lbOptions.Region = System.Drawing.Region.FromHrgn(
                CreateRoundRectRgn(0, 0,
                    this.lbOptions.Width, this.lbOptions.Height,
                    12, 12));
            this.lbExit.Region = System.Drawing.Region.FromHrgn(
                CreateRoundRectRgn(0, 0,
                    this.lbExit.Width, this.lbExit.Height,
                    12, 12));

            #endregion

			#region Collapse alarms

			this.AlarmsExpanded = true;
			this.AlarmsExpand(false);

			#endregion
		}

		#endregion

		#region Events

		#region Form

		private void frmClockMark_Load(object sender, EventArgs e)
        {
            this.dateTimeClock = new System.DateTime(((long)(0)));
        }

		private void frmClockMark_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar == (char)Keys.Escape)
			{
				this.SaveAndExit();
			}
		}

		#endregion

		#region Context Menu

		private void tsmiOptions_Click(object sender, EventArgs e)
		{
			frmOptions frmOptionsDialog = frmOptions.GetChild(this);

			if (frmOptionsDialog != null)
			{
				if (frmOptionsDialog.Visible != true)
				{
					frmOptionsDialog.ShowDialog();
				}
				else
				{
					frmOptionsDialog.Focus();
				}
			}
		}

        private void tsmiAlarms_Click(object sender, EventArgs e)
		{
			frmAlarms frmAlarmsDialog = frmAlarms.GetChild(this);
			
			if (frmAlarmsDialog != null)
			{
				if (frmAlarmsDialog.Visible != true)
				{
					frmAlarmsDialog.ShowDialog();
				}
				else
				{
					frmAlarmsDialog.Focus();
				}
			}
		}

        private void tsmiCalendar_Click(object sender, EventArgs e)
		{

		}

        private void tsmiClickThrough_Click(object sender, EventArgs e)
		{

		}

		private void tsmiUnmovable_Click(object sender, EventArgs e)
		{
			this.ClockMovementAllowed = !this.ClockMovementAllowed;
			this.tsmiUnmovable.Checked = !this.tsmiUnmovable.Checked;
		}

        private void tsmiAlwaysOnTop_Click(object sender, EventArgs e)
		{
			this.TopMost = !this.TopMost;
			this.tsmiAlwaysOnTop.Checked = !this.tsmiAlwaysOnTop.Checked;
		}

		private void tsmiAbout_Click(object sender, EventArgs e)
		{
			frmAboutBox frmAboutBoxDialog = frmAboutBox.GetChild();

			if (frmAboutBoxDialog != null)
			{
				if (frmAboutBoxDialog.Visible != true)
				{
					frmAboutBoxDialog.ShowDialog();
				}
				else
				{
					frmAboutBoxDialog.Focus();
				}
			}
		}

        private void tsmiHelp_Click(object sender, EventArgs e)
		{

		}

		private void tsmiVisit_Click(object sender, EventArgs e)
		{
            try
            {
                System.Diagnostics.Process.Start("www.sourceforge.net/clockmark");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Unable to open www.sourceforge.net/clockmark.\r\n" + ex.Message);
            }
		}

		private void tsmiExit_Click(object sender, EventArgs e)
		{
			this.SaveAndExit();
		}

		#endregion

		#region Clock

		private void pbClock_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			this.AlarmsExpand();
		}

        private void pbClock_Paint(object sender, PaintEventArgs e)
        {
            this.DrawClock(e.Graphics);
        }

        private void timerClock_Tick(object sender, EventArgs e)
        {
            this.dateTimeClock = DateTime.Now;
            this.pbClock.Invalidate();
		}

		#endregion

		#region Controls

        private void pnControl_Paint(object sender, PaintEventArgs e)
        {

        }

		private void lbAdd_Click(object sender, EventArgs e)
        {
            frmNewAlarm frmNewAlarmDialog = frmNewAlarm.GetChild(this);

            if (frmNewAlarmDialog != null)
            {
                if (frmNewAlarmDialog.Visible != true)
                {
                    frmNewAlarmDialog.ShowDialog();
                }
                else
                {
                    frmNewAlarmDialog.Focus();
                }
            }
        }

        private void lbDelete_Click(object sender, EventArgs e)
        {

        }

        private void lbList_Click(object sender, EventArgs e)
        {
            frmAlarms frmAlarmsDialog = frmAlarms.GetChild(this);

            if (frmAlarmsDialog != null)
            {
                if (frmAlarmsDialog.Visible != true)
                {
                    frmAlarmsDialog.ShowDialog();
                }
                else
                {
                    frmAlarmsDialog.Focus();
                }
            }
        }

        private void lbOptions_Click(object sender, EventArgs e)
        {
            frmOptions frmOptionsDialog = frmOptions.GetChild(this);

            if (frmOptionsDialog != null)
            {
                if (frmOptionsDialog.Visible != true)
                {
                    frmOptionsDialog.ShowDialog();
                }
                else
                {
                    frmOptionsDialog.Focus();
                }
            }
        }

        private void lbView_Click(object sender, EventArgs e)
        {

        }

        private void lbExit_Click(object sender, EventArgs e)
        {
            this.SaveAndExit();
        }

		#endregion

		#region Alarms
		private void pnAlarms_Paint(object sender, PaintEventArgs e)
		{
		}
		#endregion Alarms

		#region Move form without titlebar

		private void frmClockMark_MouseDown(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void pbClock_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void pnAlarms_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void pnControl_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void lbAdd_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void lbDelete_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void lbList_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void lbOptions_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void lbView_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		private void lbExit_MouseMove(object sender, MouseEventArgs e)
		{
			if ((this.ClockMovementAllowed) && (e.Button == MouseButtons.Left))
			{
				ReleaseCapture();
				SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
			}
		}

		#endregion

		#endregion

		#region Methods

        internal string GetIP()
        {
            string strHostName = Dns.GetHostName();
            IPHostEntry ipEntry = Dns.GetHostEntry(strHostName);
            IPAddress[] addr = ipEntry.AddressList;
            return addr[addr.Length - 1].ToString();
        }

        internal string GetIP(string server)
        {
            string strHostName = server;
            IPHostEntry ipEntry = Dns.GetHostEntry(strHostName);
            IPAddress[] addr = ipEntry.AddressList;
            return addr[addr.Length - 1].ToString();
        }

        private string[] GetUUID()
        {
            string user = (Properties.Settings.Default.Username == "Enter user") ?
                "ClockMarkUser" : Properties.Settings.Default.Username;
            
            string pass = (Properties.Settings.Default.HashedPassword == "Enter password") ?
                "ClockMarkPassword" : Properties.Settings.Default.HashedPassword;
            
            string server = (Properties.Settings.Default.Server == "Enter server") ?
                Dns.GetHostName() : Properties.Settings.Default.Server;
            
            string ip = this.GetIP(server);

            return new string[4] { user, pass, server, ip };
        }

		private Settings LoadSettings()
		{
            Settings settings = new Settings();

            settings.AutoStartLogin = Properties.Settings.Default.AutoStartLogin;
            settings.AutoStartWindows = Properties.Settings.Default.AutoStartWindows;
            settings.DesktopIcon = Properties.Settings.Default.DesktopIcon;
            settings.QuickLaunchIcon = Properties.Settings.Default.QuickLaunchIcon;
            settings.ShowInSystemTray = Properties.Settings.Default.ShowInSystemTray;
            settings.SystemTrayOnly = Properties.Settings.Default.SystemTrayOnly;
            settings.ClickThrough = Properties.Settings.Default.ClickThrough;
            settings.UnmovableWindow = Properties.Settings.Default.UnmovableWindow;
            settings.AlwaysOnTop = Properties.Settings.Default.AlwaysOnTop;

            settings.Username = Properties.Settings.Default.Username;
            settings.HashedPassword = Properties.Settings.Default.HashedPassword;
            settings.EncryptedPassword = Properties.Settings.Default.EncryptedPassword;
            settings.Server = Properties.Settings.Default.Server;
            
            settings.AutoSync = Properties.Settings.Default.AutoSync;
            settings.SyncOnShutdown = Properties.Settings.Default.SyncOnShutdown;
            settings.AskFirst = Properties.Settings.Default.AskFirst;
            settings.LastSync = Properties.Settings.Default.LastSync;

			return settings;
		}

        private void SaveSettings()
        {
            Properties.Settings.Default.Save();
        }

        private Alarms LoadAlarms()
        {
            return new Alarms(this.UUID);
        }

        private void SaveAlarms()
        {
        }

        private void SaveAndExit()
        {
            this.SaveSettings();
            this.SaveAlarms();

            if ((this.isUserLoggedIn) && (this.ClockMarkSettings.SyncOnShutdown))
            {
                if (this.ClockMarkSettings.AskFirst)
                {
                    DialogResult dr = MessageBox.Show("Synchronize the settings and alarms with the ones on the server?",
                        "Synchronize the settings and alarms?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);

                    if (dr == DialogResult.Yes)
                        this.SyncronizeWithServer(this.UUID);
                }
                else
                   this.SyncronizeWithServer(this.UUID);
            }

            Application.Exit();
        }

        internal bool IsHostname(string server)
        {
            // alphanumeric input string between 2 to 40 character long.
            return (Regex.IsMatch(server, @"^[a-zA-Z]\w{1,39}$"));
        }

        internal bool IsIP(string server)
        {
            // ip address format check
            return (Regex.IsMatch(server, @"((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)"));
        }

        private string Hash(string toHash)
        {
            byte[] data = new byte[toHash.Length];

            Encoding.UTF8.GetEncoder().GetBytes(
                toHash.ToCharArray(), 0,
                toHash.Length, data, 0, true);

            byte[] result = new SHA384Managed().ComputeHash(data);

            return BitConverter.ToString(result);
        }

        internal string HashPassword(
            string username, string password,
            int saltLevel, byte[] salt1, byte[] salt2)
        {
            if ((username == null) || (password == null))
                return null;

            string tmpPassword = null;

            switch (saltLevel)
            {
                case 2: // password + lots of salt
                    if ((salt1 == null) || (salt2 == null))
                        return null;
                    tmpPassword = Convert.ToBase64String(salt1)
                     + Convert.ToBase64String(salt2)
                     + username.ToLower() + password;
                    break;
                case 1: // user name as salt
                    tmpPassword = username.ToLower() + password;
                    break;
                case 0: // no salt
                default:
                    tmpPassword = password;
                    break;
            }

            return this.Hash(tmpPassword);
        }

        private string EncryptData(string data, out byte[] desKey, out byte[] desIV)
        {
            MemoryStream output = new MemoryStream();
            byte[] byteData = new UnicodeEncoding().GetBytes(data);

            TripleDESCryptoServiceProvider des = new TripleDESCryptoServiceProvider();
            CryptoStream crypt = new CryptoStream(output, des.CreateEncryptor(), CryptoStreamMode.Write);

            desKey = des.Key;
            desIV = des.IV;

            crypt.Write(byteData, 0, byteData.Length);

            crypt.Close();
            output.Close();

            return new UnicodeEncoding().GetString(output.ToArray());
        }

        private string DecryptData(string data, byte[] desKey, byte[] desIV)
        {
            MemoryStream output = new MemoryStream();
            byte[] byteData = new UnicodeEncoding().GetBytes(data);

            TripleDESCryptoServiceProvider des = new TripleDESCryptoServiceProvider();
            CryptoStream crypt = new CryptoStream(output, des.CreateDecryptor(desKey, desIV), CryptoStreamMode.Write);
            crypt.Write(byteData, 0, byteData.Length);

            crypt.Close();
            output.Close();

            return new UnicodeEncoding().GetString(output.ToArray());
        }

        internal string EncryptPassword(string password)
        {
            if (password != null)
               return this.EncryptData(password, out key, out iv);

           return null;
        }

        internal string DecryptPassword(string encryptedPassword)
        {
            if (encryptedPassword != null)
                return this.DecryptData(encryptedPassword, key, iv);

            return null;
        }

        private bool ConnectToTheServer(string[] uuid)
        {
            try
            {
                if ((uuid == null) ||
                    (uuid.Length != 4) ||
                    (uuid.LongLength == 0))
                {
                    System.Diagnostics.Trace.WriteLine("uuid is not correct.");
                    return false;
                }

                ClockMarkServiceClient client = new ClockMarkServiceClient(
                    "WSHttpBinding_IClockMarkService",
                    "http://" + uuid[2] + ":5555/ClockMarkService/");

                if (client != null)
                {
                    if (!client.IsServiceUp())
                    {
                        System.Diagnostics.Trace.WriteLine("Service on server " + uuid[2] + " is not ready.");
                        return false;
                    }

                    if (!client.IsExistingUser(uuid))
                    {
                        System.Diagnostics.Trace.WriteLine("user \"" + uuid[0] + "\" does not exist.");
                        return false;
                    }

                    if (!client.IsUserLoginValid(uuid))
                    {
                        System.Diagnostics.Trace.WriteLine("user's \"" + uuid[0] + "\" password is not correct.");
                        return false;
                    }

                    client.Close();
                    return true;
                }

                return false;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return false;
            }
        }

        private bool SyncronizeWithServer(string[] uuid)
        {
            try
            {
                if ((uuid == null) ||
                    (uuid.Length != 4) ||
                    (uuid.LongLength == 0))
                {
                    System.Diagnostics.Trace.WriteLine("uuid is not correct.");
                    return false;
                }

                ClockMarkServiceClient client = new ClockMarkServiceClient(
                    "WSHttpBinding_IClockMarkService",
                    "http://" + uuid[2] + ":5555/ClockMarkService/");

                if (client != null)
                {
                    if (!client.IsServiceUp())
                    {
                        System.Diagnostics.Trace.WriteLine("Service on server " + uuid[2] + " is not ready.");
                        return false;
                    }

                    if (!client.IsExistingUser(uuid))
                    {
                        System.Diagnostics.Trace.WriteLine("user \"" + uuid[0] + "\" does not exist.");
                        return false;
                    }

                    if (!client.IsUserLoginValid(uuid))
                    {
                        System.Diagnostics.Trace.WriteLine("user's \"" + uuid[0] + "\" password is not correct.");
                        return false;
                    }

                    DateTime dtSettingsOnServer = client.GetXmlFileTimeStamp(uuid, "Settings.xml");
                    DateTime dtAlarmsOnServer = client.GetXmlFileTimeStamp(uuid, "Alarms.xml");

                    if ((dtSettingsOnServer == DateTime.MinValue) ||
                        (dtAlarmsOnServer == DateTime.MinValue))
                    {
                        System.Diagnostics.Trace.WriteLine("Server " + uuid[2] + " did not return proper time stamps.");
                        return false;
                    }

                    DateTime dtSettingsOnHost = DateTime.Now;
                    DateTime dtAlarmsOnHost = System.IO.File.GetLastAccessTime(Directory.GetCurrentDirectory() +
                        Path.DirectorySeparatorChar +
                        uuid[0] +
                        "Alarms.xml");

                    string strXmlSettings;
                    string strXmlAlarms;

                    if (dtSettingsOnHost >= dtSettingsOnServer)
                    {
                        strXmlSettings = this.ClockMarkSettings.ToString();
                        if (strXmlSettings != null)
                        {
                            if (!client.PutXmlFile(uuid, "Settings.xml", strXmlSettings))
                            {
                                System.Diagnostics.Trace.WriteLine("Server " + uuid[2] + " could not put settings.");
                                return false;
                            }
                        }
                    }
                    else
                    {
                        strXmlSettings = client.GetXmlFile(uuid, "Settings.xml");
                        if (strXmlSettings == null)
                        {
                            System.Diagnostics.Trace.WriteLine("Could not get settings from server " + uuid[2] + " .");
                            return false;
                        }

                        if (!this.ClockMarkSettings.FromString(strXmlSettings))
                        {
                            System.Diagnostics.Trace.WriteLine("Settings have not been synchronized with the server " + uuid[2] + ".");
                            return false;
                        }

                        System.Diagnostics.Trace.WriteLine("Settings have been synchronized with the server " + uuid[2] + ".");
                    }

                    if (dtAlarmsOnHost >= dtAlarmsOnServer)
                    {
                        strXmlAlarms = this.ClockMarkAlarms.ToString();
                        if (strXmlAlarms != null)
                        {
                            if (!client.PutXmlFile(uuid, "Alarms.xml", strXmlAlarms))
                            {
                                System.Diagnostics.Trace.WriteLine("Server " + uuid[2] + " could not put alarms.");
                                return false;
                            }
                        }
                    }
                    else
                    {
                        strXmlAlarms = client.GetXmlFile(uuid, "Alarms.xml");
                        if (strXmlAlarms == null)
                        {
                            System.Diagnostics.Trace.WriteLine("Could not get alarms from server " + uuid[2] + " .");
                            return false;
                        }

                        if (!this.ClockMarkAlarms.FromString(strXmlAlarms))
                        {
                            System.Diagnostics.Trace.WriteLine("Alarms have not been synchronized with the server " + uuid[2] + ".");
                            return false;
                        }

                        System.Diagnostics.Trace.WriteLine("Alarms have been synchronized with the server " + uuid[2] + ".");
                    }

                    System.Diagnostics.Trace.WriteLine("User files have been synchronized with the server " + uuid[2] + ".");

                    this.ClockMarkSettings.LastSync = DateTime.Now;
                    Properties.Settings.Default.LastSync = DateTime.Now;

                    client.Close();
                    return true;
                }
                else
                {
                    System.Diagnostics.Trace.WriteLine("Server " + uuid[2] + " could not be instantied.");
                    return false;
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return false;
            }
        }

        #region Clock

        private float GetX(float radius, float deg)
		{ 
			return (float)(radius * Math.Cos((Math.PI / 180) * deg));
		}

		private float GetY(float radius, float deg)
		{
			return (float)(radius * Math.Sin((Math.PI / 180) * deg));
		}

		/// <summary>
		/// Drawing the clock itself on the graphics object passed from the Paint
		/// event of the control on which the clock is drawn
		/// </summary>
		/// <param name="graphics"></param>
		private void DrawClock(Graphics graphics)
		{
			Graphics grfx = graphics;
			grfx.SmoothingMode = SmoothingMode.AntiAlias;
			grfx.TextRenderingHint = TextRenderingHint.AntiAlias;
			grfx.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;

			midx = this.pbClock.Width / 2;
			midy = this.pbClock.Height / 2;

			x = this.pbClock.Width;
			y = this.pbClock.Height;

			SolidBrush stringBrush = new SolidBrush(this.numeralColor);
			Pen pen = new Pen(stringBrush, 2);

			
			// Define rectangles inside which we will draw circles.
			int rectBorder = 2; // 10;
			int rectRimBorder = 10; // 20;
			int rectInnerBorder = 30; // 40;
			int rectDropShadowBorder = rectBorder;
			int rectDropShadowOffset = 3;

			Rectangle rect = new Rectangle(
				0 + rectBorder, 0 + rectBorder,
				(int)x - 2*rectBorder, (int)y - 2*rectBorder);
			Rectangle rectRim = new Rectangle(
				0 + rectRimBorder, 0 + rectRimBorder,
				(int)x - 2 * rectRimBorder, (int)y - 2 * rectRimBorder);
			Rectangle rectInner = new Rectangle(
				0 + rectInnerBorder, 0 + rectInnerBorder,
				(int)x - 2 * rectInnerBorder, (int)y - 2 * rectInnerBorder);
			Rectangle rectDropShadow = new Rectangle(
				0 + rectDropShadowBorder, 0 + rectDropShadowBorder,
				(int)x - (2 * rectDropShadowBorder - rectDropShadowOffset),
				(int)y - (2 * rectDropShadowBorder - rectDropShadowOffset));

			radius = rectInner.Width / 2;

			this.fontsize = radius / 5;
			this.textFont = this.Font;

			
			// Drop Shadow
			linGradBrush = new LinearGradientBrush(rect, Color.Transparent, dropShadowColor, LinearGradientMode.BackwardDiagonal);
			rectDropShadow.Offset(dropShadowOffset);
			if (this.drawDropShadow)
				grfx.FillEllipse(linGradBrush, rectDropShadow);

			
			// Face
			linGradBrush = new LinearGradientBrush(rect, rimColor1, rimColor2, faceGradientMode);
			if (this.drawRim)
				grfx.FillEllipse(linGradBrush, rect);

			
			// Rim
			linGradBrush = new LinearGradientBrush(rect, faceColor1, faceColor2, rimGradientMode);
			grfx.FillEllipse(linGradBrush, rectRim);

			
			// Define a circular clip region and draw the image inside it.
			GraphicsPath path = new GraphicsPath();
			path.AddEllipse(rectRim);
			grfx.SetClip(path);

			if (this.imgClock != null)
				grfx.DrawImage(this.imgClock, rect);

			path.Dispose();


			// Reset clip region
			grfx.ResetClip();

			//			Triangular region
			//			pen.Width = 2;
			//			grfx.DrawRectangle(pen, rect);
			//			grfx.DrawRectangle(pen, rectinner);
			//			grfx.DrawRectangle(pen, rectrim);
			//			grfx.DrawRectangle(pen, rectdropshadow);
			//			
			//			grfx.DrawRectangle(pen, rect);
			//			grfx.DrawEllipse(pen, rect);
			//			grfx.DrawEllipse(pen, rectinner);
			//			grfx.DrawEllipse(pen, rectrim);
			//			grfx.DrawEllipse(pen, rectdropshadow);
			//			

			// Define the midpoint of the control as the centre
			grfx.TranslateTransform(midx, midy);

			// Center Point
			if (this.drawCentralPoint)
				grfx.DrawEllipse(pen, -3, -3, 6, 6);

			// Draw Numerals on the Face 
			StringFormat format = new StringFormat();
			format.Alignment = StringAlignment.Center;
			format.LineAlignment = StringAlignment.Center;
			
			int deg = 360 / 12;
			if (this.drawNumerals)
			{
				for (int i = 1; i <= 12; i++)
				{
					grfx.DrawString(
						i.ToString(),
						this.textFont,
						stringBrush,
						-1 * GetX(radius + 10, i * deg + 90),
						-1 * GetY(radius + 10, i * deg + 90),
						format);
				}
			}

			format.Dispose();

			// Set time
			hour = dateTimeClock.Hour;
			min = dateTimeClock.Minute;
			Point centre = new Point(0, 0);

			// Draw Minute hand
			if (this.drawMinuteHand)
			{
				if (minHandTickStyle == TickStyle.Smooth)
					minuteAngle = (float)(2.0 * Math.PI * (min + sec / 60.0) / 60.0);
				else
					minuteAngle = (float)(2.0 * Math.PI * (min / 60.0));

				pen.EndCap = LineCap.Round;
				pen.StartCap = LineCap.RoundAnchor;
				pen.Width = (int)radius / 14;

				centre.Offset(2, 2);
				pen.Color = Color.Gray;
				Point minHandShadow = new Point(
					(int)(radius * Math.Sin(minuteAngle)),
					(int)(-(radius) * Math.Cos(minuteAngle) + 2));
				
				if (this.drawMinuteHandShadow)
				{
					pen.Color = minuteHandDropShadowColor;
					grfx.DrawLine(pen, centre, minHandShadow);
				}

				centre.X = centre.Y = 0;
				pen.Color = minHandColor;
				Point minHand = new Point((int)(radius * Math.Sin(minuteAngle)), (int)(-(radius) * Math.Cos(minuteAngle)));
				grfx.DrawLine(pen, centre, minHand);
			}

			// Draw Hour Hand
			if (this.drawHourHand)
			{
				hourAngle = 2.0 * Math.PI * (hour + min / 60.0) / 12.0;

				pen.EndCap = LineCap.Round;
				pen.StartCap = LineCap.RoundAnchor;
				pen.Width = (int)radius / 14;

				centre.X = centre.Y = 1;
				pen.Color = Color.Gray;
				Point hourHandShadow = new Point(
					(int)((radius * Math.Sin(hourAngle) / 1.5) + 2),
					(int)((-(radius) * Math.Cos(hourAngle) / 1.5) + 2));

				if (this.drawHourHandShadow)
				{
					pen.Color = hourHandDropShadowColor;
					grfx.DrawLine(pen, centre, hourHandShadow);
				}

				centre.X = centre.Y = 0;
				pen.Color = hourHandColor;
				Point hourHand = new Point(
					(int)(radius * Math.Sin(hourAngle) / 1.5),
					(int)(-(radius) * Math.Cos(hourAngle) / 1.5));
				grfx.DrawLine(pen, centre, hourHand);
			}

			if (secondHandTickStyle == TickStyle.Smooth)
				sec = dateTimeClock.Second + (dateTimeClock.Millisecond * 0.001);
			else
				sec = dateTimeClock.Second;

			// Draw Sec Hand
			if (this.drawSecondHand)
			{
				int width = (int)radius / 25;
				pen.Width = width;
				pen.EndCap = secLineEndCap;
				pen.StartCap = LineCap.RoundAnchor;
				secondAngle = 2.0 * Math.PI * sec / 60.0;

				// Draw Second Hand Drop Shadow
				pen.Color = Color.DimGray;
				centre.X = 1;
				centre.Y = 1;

				Point secHand = new Point(
					(int)(radius * Math.Sin(secondAngle)),
					(int)(-(radius) * Math.Cos(secondAngle)));
				Point secHandshadow = new Point(
					(int)(radius * Math.Sin(secondAngle)),
					(int)(-(radius) * Math.Cos(secondAngle) + 2));

				if (this.drawSecondHandShadow)
				{
					pen.Color = secondHandDropShadowColor;
					grfx.DrawLine(pen, centre, secHandshadow);
				}

				centre.X = centre.Y = 0;
				pen.Color = secondHandColor;
				grfx.DrawLine(pen, centre, secHand);
			}

			// Draw date
			if (this.drawDate)
			{
				// Get date in proper format
				string date = dateTimeClock.Date.Day.ToString().PadLeft(2, '0');
				date += ".";
				date += dateTimeClock.Date.Month.ToString().PadLeft(2, '0');
				date += ".";
				date += dateTimeClock.Date.Year.ToString().PadLeft(4, '0');

				// Create font and brush.
				Font dateFont = new Font("Arial", 9, GraphicsUnit.Pixel);
				float dateWidth = grfx.MeasureString(date, dateFont).Width;
				SolidBrush dateBrush = new SolidBrush(Color.LightGray);
				PointF datePoint = new PointF(
					centre.X - (dateWidth / 2),
					centre.Y + (radius / 3));
				grfx.DrawString(date, dateFont, dateBrush, datePoint);
			}

			// Draw ClockMark amblem
			if (this.drawAmblem)
			{
				// Get amblem
				string amblem = "ClockMark";

				// Create font and brush.
				Font amblemFont = new Font("Arial", 9, GraphicsUnit.Pixel);
				float amblemWidth = grfx.MeasureString(amblem, amblemFont).Width;
				float amblemHeight = grfx.MeasureString(amblem, amblemFont).Height;
				SolidBrush amblemBrush = new SolidBrush(Color.LightGray);
				PointF amblemPoint = new PointF(
					centre.X - (amblemWidth / 2),
					centre.Y - radius + amblemHeight / 2);
				grfx.DrawString(amblem, amblemFont, amblemBrush, amblemPoint);
			}

			pen.Dispose();
		}

		private void AlarmsExpand(bool expand)
		{
			if (expand)
			{
				if (!this.AlarmsExpanded)
				{
					this.Height += this.pnAlarms.Height;
					this.AlarmsExpanded = true;
				}
			}
			else
			{
				if (this.AlarmsExpanded)
				{
					this.Height -= this.pnAlarms.Height;
					this.AlarmsExpanded = false;
				}
			}
			
			this.Region = System.Drawing.Region.FromHrgn(
				CreateRoundRectRgn(0, 0,
					this.Width, this.Height,
					20, 20));	
		}

		private void AlarmsExpand()
		{
			if (!this.AlarmsExpanded)
			{
				this.Height += this.pnAlarms.Height;
			}
			else
			{
				this.Height -= this.pnAlarms.Height;
			}
			this.AlarmsExpanded = !this.AlarmsExpanded;
			this.Invalidate();
			this.Update();

			this.Region = System.Drawing.Region.FromHrgn(
				CreateRoundRectRgn(0, 0,
					this.Width, this.Height,
					20, 20));
        }

        #endregion

		#endregion
	}
}