﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Net.NetworkInformation;
using System.Globalization;
using Microsoft.Win32; // for registry access
using System.IO; // for creating shortcuts
using IWshRuntimeLibrary; // for creating shortcuts

namespace ClockMark
{
	public partial class frmOptions : Form
	{
		private enum InformTheUserType
		{
			ERROR = 0,
			WARNING,
			INFO
		};

        private enum InformTheUserLocation
        {
            GENERAL_OPTIONS = 0,
            SERVER_OPTIONS
        };

		private enum AutoStart
		{
			USER_LOGIN = 0,
			WINDOWS_START
		}

		private enum Shortcut
		{
			DESKTOP = 0,
			QUICK_LAUNCH
        }

        private frmClockMark frmParent;
        private static frmOptions _childInstance = null;

        #region Constructor

        private frmOptions(frmClockMark frmParent)
		{
			InitializeComponent();

            this.frmParent = frmParent;
			this.Icon = Properties.Resources.ClockMark;

			// load the saved settings from disk
			//this.LoadSettings();
			// load and sync the settings from the actual system to the options dialog
			this.CheckFirstThanLoadSettings();

            if (this.frmParent.IsUserLoggedIn)
            {
                this.InformTheUser(
                    InformTheUserType.INFO,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Logged in.");

                this.cbAutoSync.Enabled = true;
                this.cbSyncOnShutdown.Enabled = true;
                this.cbAskFirst.Enabled = true;
                this.btSychronize.Enabled = true;
                this.btDownload.Enabled = true;
                this.btUpload.Enabled = true;
            }
            else
            {
                this.cbAutoSync.Enabled = false;
                this.cbSyncOnShutdown.Enabled = false;
                this.cbAskFirst.Enabled = false;
                this.btSychronize.Enabled = false;
                this.btDownload.Enabled = false;
                this.btUpload.Enabled = false;

                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Not logged in!");
            }
		}

        public static frmOptions GetChild(frmClockMark frmParent)
        {
            if (_childInstance == null)
                _childInstance = new frmOptions(frmParent);

            return _childInstance;
        }

        #endregion 

        #region Events

        // general options
        private void cbAutoStartUser_CheckedChanged(object sender, EventArgs e)
        {
			this.UpdateAutoRun(
				AutoStart.USER_LOGIN,
				Application.ExecutablePath,
				"ClockMark",
				this.cbAutoStartUser.Checked);

			Properties.Settings.Default.AutoStartLogin = this.cbAutoStartUser.Checked;
            this.frmParent.ClockMarkSettings.AutoStartLogin = this.cbAutoStartUser.Checked;
        }

        private void cbAutoStartWindows_CheckedChanged(object sender, EventArgs e)
        {
			this.UpdateAutoRun(
				AutoStart.WINDOWS_START,
				Application.ExecutablePath,
				"ClockMark",
				this.cbAutoStartWindows.Checked);

			Properties.Settings.Default.AutoStartWindows = this.cbAutoStartWindows.Checked;
            this.frmParent.ClockMarkSettings.AutoStartWindows = this.cbAutoStartWindows.Checked;
        }

        private void cbDesktopIcon_CheckedChanged(object sender, EventArgs e)
        {
            this.UpdateShortcut(
				Shortcut.DESKTOP,
                Application.ExecutablePath,
                "ClockMark",
                this.cbDesktopIcon.Checked);

			Properties.Settings.Default.DesktopIcon = this.cbDesktopIcon.Checked;
            this.frmParent.ClockMarkSettings.DesktopIcon = this.cbDesktopIcon.Checked;
        }

        private void cbQuickLaunchIcon_CheckedChanged(object sender, EventArgs e)
        {
            this.UpdateShortcut(
                Shortcut.QUICK_LAUNCH,
                Application.ExecutablePath,
                "ClockMark",
                this.cbQuickLaunchIcon.Checked);

			Properties.Settings.Default.QuickLaunchIcon = this.cbQuickLaunchIcon.Checked;
            this.frmParent.ClockMarkSettings.QuickLaunchIcon = this.cbQuickLaunchIcon.Checked;
        }

        private void cbSystemTrayIcon_CheckedChanged(object sender, EventArgs e)
        {
			Properties.Settings.Default.ShowInSystemTray = this.cbShowInSystemTray.Checked;
            this.frmParent.ClockMarkSettings.ShowInSystemTray = this.cbShowInSystemTray.Checked;
        }

        private void cbSystemTrayOnly_CheckedChanged(object sender, EventArgs e)
        {
            this.frmParent.ShowInTaskbar = !this.cbSystemTrayOnly.Checked;
            
			Properties.Settings.Default.SystemTrayOnly = this.cbSystemTrayOnly.Checked;
            this.frmParent.ClockMarkSettings.SystemTrayOnly = this.cbSystemTrayOnly.Checked;
        }

        private void cbClickThrough_CheckedChanged(object sender, EventArgs e)
        {
			Properties.Settings.Default.ClickThrough = this.cbClickThrough.Checked;
            this.frmParent.ClockMarkSettings.ClickThrough = this.cbClickThrough.Checked;
        }

        private void cbUnmovable_CheckedChanged(object sender, EventArgs e)
        {
			Properties.Settings.Default.UnmovableWindow = this.cbUnmovableWindow.Checked;
            this.frmParent.ClockMarkSettings.UnmovableWindow = this.cbUnmovableWindow.Checked;
        }

        private void cbOnTop_CheckedChanged(object sender, EventArgs e)
        {
			Properties.Settings.Default.AlwaysOnTop = this.cbAlwaysOnTop.Checked;
            this.frmParent.ClockMarkSettings.AlwaysOnTop = this.cbAlwaysOnTop.Checked;
        }

        private void btDefaults_Click(object sender, EventArgs e)
        {
			this.SetDefaultSettings();
			this.SaveSettings();
        }

        // server options
        private void tbUsername_Click(object sender, EventArgs e)
        {
            this.tbUsername.SelectAll();
        }

        private void tbUsername_Enter(object sender, EventArgs e)
        {
            this.tbUsername.SelectAll();
        }

		private void tbUsername_TextChanged(object sender, EventArgs e)
		{
			this.ClearInfoArea(InformTheUserLocation.SERVER_OPTIONS);
			Properties.Settings.Default.Username = this.tbUsername.Text;
            this.frmParent.ClockMarkSettings.Username = this.tbUsername.Text;
		}

        private void tbPassword_Click(object sender, EventArgs e)
        {
            this.tbPassword.SelectAll();
        }

        private void tbPassword_Enter(object sender, EventArgs e)
        {
            this.tbPassword.SelectAll();
        }

		private void tbPassword_TextChanged(object sender, EventArgs e)
		{
            this.ClearInfoArea(InformTheUserLocation.SERVER_OPTIONS);

            if ((this.tbUsername.Text.Length == 0) ||
                (this.tbUsername.Text.Length > 14) ||
                (this.tbUsername.Text.CompareTo("Enter username") == 0) ||
                (this.tbUsername.Text.IndexOfAny(new char[] { ' ', '@', '.', ',', '!' }) != -1))
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "First enter the username!");

                this.tbPassword.Text = "Enter password";

                return;
            }

            string hashPassword = this.frmParent.HashPassword(
                this.tbUsername.Text.Trim(),
                this.tbPassword.Text.Trim(),
                1, null, null);
			Properties.Settings.Default.HashedPassword = hashPassword;
            this.frmParent.ClockMarkSettings.HashedPassword = hashPassword;

            string encryptedPassword = this.frmParent.EncryptPassword(this.tbPassword.Text.Trim());
            Properties.Settings.Default.EncryptedPassword = encryptedPassword;
            this.frmParent.ClockMarkSettings.EncryptedPassword = encryptedPassword;
		}

        private void tbServer_Click(object sender, EventArgs e)
        {
            this.tbServer.SelectAll();
        }

        private void tbServer_Enter(object sender, EventArgs e)
        {
            this.tbServer.SelectAll();
        }

		private void tbServer_TextChanged(object sender, EventArgs e)
		{
            this.ClearInfoArea(InformTheUserLocation.SERVER_OPTIONS);
			Properties.Settings.Default.Server = this.tbServer.Text;
            this.frmParent.ClockMarkSettings.Server = this.tbServer.Text;
		}

		private void btCheckServer_Click(object sender, EventArgs e)
		{
            if ((this.tbServer.Text.Length == 0) ||
                (!this.frmParent.IsIP(this.tbServer.Text) && !this.frmParent.IsHostname(this.tbServer.Text)) ||
                (this.tbServer.Text.CompareTo("Enter server") == 0) ||
                (this.tbServer.Text.IndexOfAny(new char[] { ' ', '@', ',', '!', '=' }) != -1))
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Server must not be empty, has to be either in ip or hostname format, " +
                    " and must not contain spaces or other interpunction characters.");

                return;
            }

			this.btCheckServer.Enabled = false;

            // first, primitive way, disposed
            /*this.PingServerAsync(this.tbServer.Text);

			this.InformTheUser(
				InformTheUserType.INFO,
                InformTheUserLocation.SERVER_OPTIONS,
				"Pinging " + this.tbServer.Text + "...");*/

            // second, wcf way

            try
            {
                ClockMarkServiceClient client = new ClockMarkServiceClient(
                    "WSHttpBinding_IClockMarkService",
                    "http://" + this.tbServer.Text.Trim() + ":5555/ClockMarkService/");

                if (client != null)
                {
                    if (client.IsServiceUp())
                    {
                        this.InformTheUser(
                            InformTheUserType.INFO,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "Server " + this.tbServer.Text + " is up and service is ready.");
                        
                        client.Close();
                        return;
                    }
                }

                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Server " + this.tbServer.Text + " is NOT up and service is NOT ready.");
            }
            catch (Exception ex)
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    ex.Message);
            }
            finally
            {
                this.btCheckServer.Enabled = true;
            }
		}

        private void btLogin_Click(object sender, EventArgs e)
        {
            if ((this.tbUsername.Text.Length == 0) ||
                (this.tbUsername.Text.Length > 14) ||
                (this.tbUsername.Text.CompareTo("Enter username") == 0) ||
                (this.tbUsername.Text.IndexOfAny(new char[] { ' ', '@', '.', ',', '!' }) != -1))
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Username must not be empty, longer than 14 characters or " +
                    "containing spaces or other interpunction characters.");

                return;
            }

            if ((this.tbPassword.Text.Length == 0) ||
                (this.tbPassword.Text.Length > 14) ||
                (this.tbPassword.Text.CompareTo("Enter password") == 0) ||
                (this.tbPassword.Text.IndexOfAny(new char[] { ' ', '@', '.', ',', '!' }) != -1))
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Password must not be empty, longer than 14 characters or " +
                    "containing spaces or other interpunction characters.");

                return;
            }

            if ((this.tbServer.Text.Length == 0) ||
                (!this.frmParent.IsIP(this.tbServer.Text) && !this.frmParent.IsHostname(this.tbServer.Text)) ||
                (this.tbServer.Text.CompareTo("Enter server") == 0) ||
                (this.tbServer.Text.IndexOfAny(new char[] { ' ', '@', ',', '!', '=' }) != -1))
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Server must not be empty, has to be either in ip or hostname format, " +
                    " and must not contain spaces or other interpunction characters.");

                return;
            }

            this.InformTheUser(
                InformTheUserType.INFO,
                InformTheUserLocation.SERVER_OPTIONS,
                "Trying to log to server...");

            if (this.ConnectToTheServer())
            {
                this.InformTheUser(
                    InformTheUserType.INFO,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Connection established.");

                this.cbAutoSync.Enabled = true;
                this.cbSyncOnShutdown.Enabled = true;
                this.cbAskFirst.Enabled = true;
                this.btSychronize.Enabled = true;
                this.btDownload.Enabled = true;
                this.btUpload.Enabled = true;
            }
            else
            {
                this.cbAutoSync.Enabled = false;
                this.cbSyncOnShutdown.Enabled = false;
                this.cbAskFirst.Enabled = false;
                this.btSychronize.Enabled = false;
                this.btDownload.Enabled = false;
                this.btUpload.Enabled = false;

                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Connection could not be established!");
            }
        }

		private void cbAutomaticSynchronization_CheckedChanged(object sender, EventArgs e)
		{
			Properties.Settings.Default.AutoSync = this.cbAutoSync.Checked;
            this.frmParent.ClockMarkSettings.AutoSync = this.cbAutoSync.Checked;
		}

		private void cbSyncronizeOnShutdown_CheckedChanged(object sender, EventArgs e)
		{
			Properties.Settings.Default.SyncOnShutdown = this.cbSyncOnShutdown.Checked;
            this.frmParent.ClockMarkSettings.SyncOnShutdown = this.cbSyncOnShutdown.Checked;
		}

		private void cbAskFirst_CheckedChanged(object sender, EventArgs e)
		{
			Properties.Settings.Default.AskFirst = this.cbAskFirst.Checked;
            this.frmParent.ClockMarkSettings.AskFirst = this.cbAskFirst.Checked;
		}

		private void btSychronize_Click(object sender, EventArgs e)
		{
            this.btSychronize.Enabled = false;

            try
            {
                ClockMarkServiceClient client = new ClockMarkServiceClient(
                    "WSHttpBinding_IClockMarkService",
                    "http://" + this.tbServer.Text.Trim() + ":5555/ClockMarkService/");

                if (client != null)
                {
                    if (!client.IsServiceUp())
                    {
                        this.InformTheUser(
                            InformTheUserType.ERROR,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "Server " + this.tbServer.Text + " is NOT up and service is NOT ready.");
                        return;
                    }

                    DateTime dtSettingsOnServer = client.GetXmlFileTimeStamp(this.frmParent.UUID, "Settings.xml");
                    DateTime dtAlarmsOnServer = client.GetXmlFileTimeStamp(this.frmParent.UUID, "Alarms.xml");

                    if ((dtSettingsOnServer == DateTime.MinValue) ||
                        (dtAlarmsOnServer == DateTime.MinValue))
                    {
                        this.InformTheUser(
                            InformTheUserType.ERROR,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "Server " + this.tbServer.Text + " did not return proper time stamps.");
                        return;
                    }

                    DateTime dtSettingsOnHost = DateTime.Now;
                    DateTime dtAlarmsOnHost = System.IO.File.GetLastAccessTime(Directory.GetCurrentDirectory() +
                        Path.DirectorySeparatorChar +
                        this.frmParent.UUID[0] + 
                        "Alarms.xml");

                    string strXmlSettings;
                    string strXmlAlarms;

                    if (dtSettingsOnHost >= dtSettingsOnServer)
                    {
                        strXmlSettings = this.frmParent.ClockMarkSettings.ToString();
                        if (strXmlSettings != null)
                        {
                            if (!client.PutXmlFile(this.frmParent.UUID, "Settings.xml", strXmlSettings))
                            {
                                this.InformTheUser(
                                    InformTheUserType.ERROR,
                                    InformTheUserLocation.SERVER_OPTIONS,
                                    "Server " + this.tbServer.Text + " could not put settings.");
                                return;
                            }
                        }
                    }
                    else
                    {
                        strXmlSettings = client.GetXmlFile(this.frmParent.UUID, "Settings.xml");
                        if (strXmlSettings == null)
                        {
                            this.InformTheUser(
                                InformTheUserType.ERROR,
                                InformTheUserLocation.SERVER_OPTIONS,
                                "Could not get settings from server " + this.tbServer.Text + " .");
                            return;
                        }

                        if (this.frmParent.ClockMarkSettings.FromString(strXmlSettings))
                        {
                            this.InformTheUser(
                                InformTheUserType.INFO,
                                InformTheUserLocation.SERVER_OPTIONS,
                                "Settings have been synchronized with the server " + this.tbServer.Text + ".");
                        }
                    }

                    if (dtAlarmsOnHost >= dtAlarmsOnServer)
                    {
                        strXmlAlarms = this.frmParent.ClockMarkAlarms.ToString();
                        if (strXmlAlarms != null)
                        {
                            if (!client.PutXmlFile(this.frmParent.UUID, "Alarms.xml", strXmlAlarms))
                            {
                                this.InformTheUser(
                                    InformTheUserType.ERROR,
                                    InformTheUserLocation.SERVER_OPTIONS,
                                    "Server " + this.tbServer.Text + " could not put alarms.");
                                return;
                            }
                        }
                    }
                    else
                    {
                        strXmlAlarms = client.GetXmlFile(this.frmParent.UUID, "Alarms.xml");
                        if (strXmlAlarms == null)
                        {
                            this.InformTheUser(
                                InformTheUserType.ERROR,
                                InformTheUserLocation.SERVER_OPTIONS,
                                "Could not get alarms from server " + this.tbServer.Text + " .");
                            return;
                        }

                        if (this.frmParent.ClockMarkAlarms.FromString(strXmlAlarms))
                        {
                            this.InformTheUser(
                                InformTheUserType.INFO,
                                InformTheUserLocation.SERVER_OPTIONS,
                                "Alarms have been synchronized with the server " + this.tbServer.Text + ".");
                        }
                    }

                    this.InformTheUser(
                        InformTheUserType.INFO,
                        InformTheUserLocation.SERVER_OPTIONS,
                        "User files have been synchronized with the server " + this.tbServer.Text + ".");

                    this.lbLastSync.Text = DateTime.Now.ToString();
                    this.frmParent.ClockMarkSettings.LastSync = DateTime.Now;
                    Properties.Settings.Default.LastSync = DateTime.Now;

                    client.Close();
                }
                else
                {
                    this.InformTheUser(
                        InformTheUserType.ERROR,
                        InformTheUserLocation.SERVER_OPTIONS,
                        "Server " + this.tbServer.Text + " could not be instantied.");
                    return;
                }
            }
            catch (Exception ex)
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    ex.Message);
            }
            finally
            {
                this.btSychronize.Enabled = true;
            }
		}

		private void btDownload_Click(object sender, EventArgs e)
		{
            this.btDownload.Enabled = false;

            try
            {
                ClockMarkServiceClient client = new ClockMarkServiceClient(
                    "WSHttpBinding_IClockMarkService",
                    "http://" + this.tbServer.Text.Trim() + ":5555/ClockMarkService/");

                if (client != null)
                {
                    if (!client.IsServiceUp())
                    {
                        this.InformTheUser(
                            InformTheUserType.ERROR,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "Server " + this.tbServer.Text + " is NOT up and service is NOT ready.");
                        return;
                    }

                    string strXmlSettings = client.GetXmlFile(this.frmParent.UUID, "Settings.xml");
                    if (strXmlSettings == null)
                    {
                        this.InformTheUser(
                            InformTheUserType.ERROR,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "Could not get settings from server " + this.tbServer.Text + " .");
                        return;
                    }

                    string strXmlAlarms = client.GetXmlFile(this.frmParent.UUID, "Alarms.xml");
                    if (strXmlAlarms == null)
                    {
                        this.InformTheUser(
                            InformTheUserType.ERROR,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "Could not get alarms from server " + this.tbServer.Text + " .");
                        return;
                    }

                    if (this.frmParent.ClockMarkSettings.FromString(strXmlSettings) &&
                        this.frmParent.ClockMarkAlarms.FromString(strXmlAlarms))
                    {
                        this.InformTheUser(
                            InformTheUserType.INFO,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "User files have been downloaded from server " + this.tbServer.Text + ".");
                    }

                    client.Close();
                }
                else
                {
                    this.InformTheUser(
                        InformTheUserType.ERROR,
                        InformTheUserLocation.SERVER_OPTIONS,
                        "Server " + this.tbServer.Text + " could not be instantied.");
                    return;
                }
            }
            catch (Exception ex)
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    ex.Message);
            }
            finally
            {
                this.btDownload.Enabled = true;
            }
		}

		private void btUpload_Click(object sender, EventArgs e)
		{
            this.btUpload.Enabled = false;

            try
            {
                ClockMarkServiceClient client = new ClockMarkServiceClient(
                    "WSHttpBinding_IClockMarkService",
                    "http://" + this.tbServer.Text.Trim() + ":5555/ClockMarkService/");

                if (client != null)
                {
                    if (!client.IsServiceUp())
                    {
                        this.InformTheUser(
                            InformTheUserType.ERROR,
                            InformTheUserLocation.SERVER_OPTIONS,
                            "Server " + this.tbServer.Text + " is NOT up and service is NOT ready.");
                        return;
                    }

                    string strXmlSettings = this.frmParent.ClockMarkSettings.ToString();
                    if (strXmlSettings != null)
                    {
                        if (!client.PutXmlFile(this.frmParent.UUID, "Settings.xml", strXmlSettings))
                        {
                            this.InformTheUser(
                                InformTheUserType.ERROR,
                                InformTheUserLocation.SERVER_OPTIONS,
                                "Server " + this.tbServer.Text + " could not put settings.");
                            return;
                        }
                    }

                    string strXmlAlarms = this.frmParent.ClockMarkAlarms.ToString();
                    if (strXmlAlarms != null)
                    {
                        if (!client.PutXmlFile(this.frmParent.UUID, "Alarms.xml", strXmlAlarms))
                        {
                            this.InformTheUser(
                                InformTheUserType.ERROR,
                                InformTheUserLocation.SERVER_OPTIONS,
                                "Server " + this.tbServer.Text + " could not put alarms.");
                            return;
                        }
                    }

                    this.InformTheUser(
                        InformTheUserType.INFO,
                        InformTheUserLocation.SERVER_OPTIONS,
                        "User files have been sent to server " + this.tbServer.Text + ".");

                    client.Close();
                }
                else
                {
                    this.InformTheUser(
                        InformTheUserType.ERROR,
                        InformTheUserLocation.SERVER_OPTIONS,
                        "Server " + this.tbServer.Text + " could not be instantied.");
                    return;
                }
            }
            catch (Exception ex)
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    ex.Message);
            }
            finally
            {
                this.btUpload.Enabled = true;
            }
		}

        // options
        private void btOK_Click(object sender, EventArgs e)
        {
            this.SaveSettings();
            this.Close();
        }

        private void btCancel_Click(object sender, EventArgs e)
        {
            this.Close();
		}

		#endregion
        
		#region Methods

		private void LoadSettings()
		{
			// general
			this.cbAutoStartUser.Checked = Properties.Settings.Default.AutoStartLogin;
			this.cbAutoStartWindows.Checked = Properties.Settings.Default.AutoStartWindows;
			this.cbDesktopIcon.Checked = Properties.Settings.Default.DesktopIcon;
			this.cbQuickLaunchIcon.Checked = Properties.Settings.Default.QuickLaunchIcon;
			this.cbShowInSystemTray.Checked = Properties.Settings.Default.ShowInSystemTray;
			this.cbSystemTrayOnly.Checked = Properties.Settings.Default.SystemTrayOnly;
			this.cbClickThrough.Checked = Properties.Settings.Default.ClickThrough;
			this.cbUnmovableWindow.Checked = Properties.Settings.Default.UnmovableWindow;
			this.cbAlwaysOnTop.Checked = Properties.Settings.Default.AlwaysOnTop;

			// server
			this.tbUsername.Text = Properties.Settings.Default.Username;
            this.tbPassword.Text = String.Empty; //Properties.Settings.Default.Password;
			this.tbServer.Text = Properties.Settings.Default.Server;
			this.cbAutoSync.Checked = Properties.Settings.Default.AutoSync;
			this.cbSyncOnShutdown.Checked = Properties.Settings.Default.SyncOnShutdown;
			this.cbAskFirst.Checked = Properties.Settings.Default.AskFirst;
		}

		private void CheckFirstThanLoadSettings()
		{
			// general
			if (this.IsAutoRunSet(AutoStart.USER_LOGIN))
			{
				this.cbAutoStartUser.Checked = true;
				Properties.Settings.Default.AutoStartLogin = true;
                this.frmParent.ClockMarkSettings.AutoStartLogin = true;
			}
			else
			{
				this.cbAutoStartUser.Checked = false;
				Properties.Settings.Default.AutoStartLogin = false;
                this.frmParent.ClockMarkSettings.AutoStartLogin = false;
			}

			if (this.IsAutoRunSet(AutoStart.WINDOWS_START))
			{
				this.cbAutoStartWindows.Checked = true;
				Properties.Settings.Default.AutoStartWindows = true;
                this.frmParent.ClockMarkSettings.AutoStartWindows = true;
			}
			else
			{
				this.cbAutoStartWindows.Checked = false;
				Properties.Settings.Default.AutoStartWindows = false;
                this.frmParent.ClockMarkSettings.AutoStartWindows = false;
			}

			if (this.IsShortcutSet(Shortcut.DESKTOP, "ClockMark"))
			{
				this.cbDesktopIcon.Checked = true;
				Properties.Settings.Default.DesktopIcon = true;
                this.frmParent.ClockMarkSettings.DesktopIcon = true;
			}
			else
			{
				this.cbDesktopIcon.Checked = false;
				Properties.Settings.Default.DesktopIcon = false;
                this.frmParent.ClockMarkSettings.DesktopIcon = false;
			}

			if (this.IsShortcutSet(Shortcut.QUICK_LAUNCH, "ClockMark"))
			{
				this.cbQuickLaunchIcon.Checked = true;
				Properties.Settings.Default.QuickLaunchIcon = true;
                this.frmParent.ClockMarkSettings.QuickLaunchIcon = true;
			}
			else
			{
				this.cbQuickLaunchIcon.Checked = false;
				Properties.Settings.Default.QuickLaunchIcon = false;
                this.frmParent.ClockMarkSettings.QuickLaunchIcon = false;
			}
			
			this.cbShowInSystemTray.Checked = Properties.Settings.Default.ShowInSystemTray;
            this.frmParent.ClockMarkSettings.ShowInSystemTray = Properties.Settings.Default.ShowInSystemTray;

			this.cbSystemTrayOnly.Checked = Properties.Settings.Default.SystemTrayOnly;
            this.frmParent.ClockMarkSettings.SystemTrayOnly = Properties.Settings.Default.SystemTrayOnly;

			this.cbClickThrough.Checked = Properties.Settings.Default.ClickThrough;
            this.frmParent.ClockMarkSettings.ClickThrough = Properties.Settings.Default.ClickThrough;

			this.cbUnmovableWindow.Checked = Properties.Settings.Default.UnmovableWindow;
            this.frmParent.ClockMarkSettings.UnmovableWindow = Properties.Settings.Default.UnmovableWindow;

			this.cbAlwaysOnTop.Checked = Properties.Settings.Default.AlwaysOnTop;
            this.frmParent.ClockMarkSettings.AlwaysOnTop = Properties.Settings.Default.AlwaysOnTop;

			// server
			this.tbUsername.Text = Properties.Settings.Default.Username;
            this.frmParent.ClockMarkSettings.Username = Properties.Settings.Default.Username;

            //this.tbPassword.Text = this.frmParent.DecryptPassword(Properties.Settings.Default.EncryptedPassword);
            this.frmParent.ClockMarkSettings.HashedPassword = Properties.Settings.Default.HashedPassword;
            this.frmParent.ClockMarkSettings.EncryptedPassword = Properties.Settings.Default.EncryptedPassword;
			
            this.tbServer.Text = Properties.Settings.Default.Server;
            this.frmParent.ClockMarkSettings.Server = Properties.Settings.Default.Server;
			
            this.cbAutoSync.Checked = Properties.Settings.Default.AutoSync;
            this.frmParent.ClockMarkSettings.AutoSync = Properties.Settings.Default.AutoSync;
			
            this.cbSyncOnShutdown.Checked = Properties.Settings.Default.SyncOnShutdown;
            this.frmParent.ClockMarkSettings.SyncOnShutdown = Properties.Settings.Default.SyncOnShutdown;
			
            this.cbAskFirst.Checked = Properties.Settings.Default.AskFirst;
            this.frmParent.ClockMarkSettings.AlwaysOnTop = Properties.Settings.Default.AlwaysOnTop;

            this.lbLastSync.Text = Properties.Settings.Default.LastSync.ToString();
            this.frmParent.ClockMarkSettings.LastSync = Properties.Settings.Default.LastSync;
		}

		private void SetDefaultSettings()
		{
			// general
			this.cbAutoStartUser.Checked = true;
            this.frmParent.ClockMarkSettings.AutoStartLogin = true;

			this.cbAutoStartWindows.Checked = false;
            this.frmParent.ClockMarkSettings.AutoStartWindows = false;

			this.cbDesktopIcon.Checked = false;
            this.frmParent.ClockMarkSettings.DesktopIcon = false;

			this.cbQuickLaunchIcon.Checked = true;
            this.frmParent.ClockMarkSettings.QuickLaunchIcon = false;

			this.cbShowInSystemTray.Checked = true;
            this.frmParent.ClockMarkSettings.ShowInSystemTray = true;

			this.cbSystemTrayOnly.Checked = true;
            this.frmParent.ClockMarkSettings.SystemTrayOnly = true;

			this.cbClickThrough.Checked = false;
            this.frmParent.ClockMarkSettings.ClickThrough = false;

            this.cbUnmovableWindow.Checked = false;
            this.frmParent.ClockMarkSettings.UnmovableWindow = false;

            this.cbAlwaysOnTop.Checked = true;
            this.frmParent.ClockMarkSettings.AlwaysOnTop = true;

			// server
			this.tbUsername.Text = Properties.Settings.Default.Username;
            this.frmParent.ClockMarkSettings.Username = Properties.Settings.Default.Username;

            this.tbPassword.Text = String.Empty; //Properties.Settings.Default.Password;
            this.frmParent.ClockMarkSettings.HashedPassword = Properties.Settings.Default.HashedPassword;

			this.tbServer.Text = Properties.Settings.Default.Server;
            this.frmParent.ClockMarkSettings.Server = Properties.Settings.Default.Server;

			this.cbAutoSync.Checked = true;
            this.frmParent.ClockMarkSettings.AutoSync = true;

			this.cbSyncOnShutdown.Checked = true;
            this.frmParent.ClockMarkSettings.SyncOnShutdown = true;

			this.cbAskFirst.Checked = false;
            this.frmParent.ClockMarkSettings.AskFirst = false;

            this.lbLastSync.Text = Properties.Settings.Default.LastSync.ToString();
            this.frmParent.ClockMarkSettings.LastSync = Properties.Settings.Default.LastSync;
		}

		private void SetSettings()
		{
			// general
			Properties.Settings.Default.AutoStartLogin = this.cbAutoStartUser.Checked;
			Properties.Settings.Default.AutoStartWindows = this.cbAutoStartWindows.Checked;
			Properties.Settings.Default.DesktopIcon = this.cbDesktopIcon.Checked;
			Properties.Settings.Default.QuickLaunchIcon = this.cbQuickLaunchIcon.Checked;
			Properties.Settings.Default.ShowInSystemTray = this.cbShowInSystemTray.Checked;
			Properties.Settings.Default.SystemTrayOnly = this.cbSystemTrayOnly.Checked;
			Properties.Settings.Default.ClickThrough = this.cbClickThrough.Checked;
			Properties.Settings.Default.UnmovableWindow = this.cbUnmovableWindow.Checked;
			Properties.Settings.Default.AlwaysOnTop = this.cbAlwaysOnTop.Checked;

			// server
			Properties.Settings.Default.Username = this.tbUsername.Text;
			//Properties.Settings.Default.Password = this.tbPassword.Text;
			Properties.Settings.Default.Server = this.tbServer.Text;
			Properties.Settings.Default.AutoSync = this.cbAutoSync.Checked;
			Properties.Settings.Default.SyncOnShutdown = this.cbSyncOnShutdown.Checked;
			Properties.Settings.Default.AskFirst = this.cbAskFirst.Checked;
            Properties.Settings.Default.LastSync = DateTime.Parse(this.lbLastSync.Text);
		}

		private void SaveSettings()
		{
			Properties.Settings.Default.Save();
		}

		private void ClearInfoArea(InformTheUserLocation iLocation)
		{
            switch (iLocation)
            {
                case InformTheUserLocation.GENERAL_OPTIONS:
                    this.lbInfoGeneral.Text = "";
                    this.lbInfoGeneral.ForeColor = Color.FromName("ControlText");
                    this.lbInfoGeneral.Font = new Font(this.lbInfoGeneral.Font, FontStyle.Regular);
                    break;
                case InformTheUserLocation.SERVER_OPTIONS:
                    this.lbInfoServer.Text = "";
                    this.lbInfoServer.ForeColor = Color.FromName("ControlText");
                    this.lbInfoServer.Font = new Font(this.lbInfoServer.Font, FontStyle.Regular);
                    break;
                default:
                    break;
            }
		}

		private void InformTheUser(InformTheUserType iType, InformTheUserLocation iLocation, string strMessage)
		{
            // set area to clean state
            this.ClearInfoArea(iLocation);

			// compose information
			string strInfo;
			string strInfoHeader;

            System.Windows.Forms.Label lbTemp = this.lbInfoGeneral;

            switch (iLocation)
            {
                case InformTheUserLocation.GENERAL_OPTIONS:
                    lbTemp = this.lbInfoGeneral;
                    break;
                case InformTheUserLocation.SERVER_OPTIONS:
                    lbTemp = this.lbInfoServer;
                    break;
                default:
                    break;
            }

			switch (iType)
			{
				case InformTheUserType.ERROR:
					strInfoHeader = "Error";
                    lbTemp.ForeColor = Color.Red;
                    lbTemp.Font = new Font(lbTemp.Font, FontStyle.Bold);
					break;
				case InformTheUserType.WARNING:
					strInfoHeader = "Warning";
                    lbTemp.ForeColor = Color.Orange;
                    lbTemp.Font = new Font(lbTemp.Font, FontStyle.Bold);
					break;
				case InformTheUserType.INFO:
                    strInfoHeader = "Info";
                    lbTemp.ForeColor = Color.Navy;
                    break;
				default:
					strInfoHeader = "Info";
                    lbTemp.ForeColor = Color.Navy;
					break;
			}

			strInfo = strInfoHeader + ": " + strMessage;

			// present information to the user
			// 1. classic popup messagebox (OUT-DATED)
			/*MessageBox.Show(
				strInfo, strInfoHeader,
				MessageBoxButtons.OK, MessageBoxIcon.Information);*/

			// 2. New non-obstructive way (UP-TO-DATE)
            lbTemp.Text = strInfo;
		}

		/// <summary>
		/// Check if the autorun is set for ClockMark
		/// </summary>
		/// <param name="asAt">at user login or windows login</param>
		/// <returns>it is set or not to autorun</returns>
		private bool IsAutoRunSet(AutoStart asAt)
		{
			RegistryKey rkRun;

			switch (asAt)
			{
				case AutoStart.USER_LOGIN:
					rkRun = Registry.CurrentUser.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", true);
					break;
				case AutoStart.WINDOWS_START:
					rkRun = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", true);
					break;
				default:
					rkRun = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", true);
					break;
			}
			
			return (rkRun.GetValue("ClockMark") != null);
		}

		/// <summary>
		/// Update the registry to start the ClockMark at user login
		/// </summary>
		/// <param name="asAt">at user login or windows login</param>
		/// <param name="start">Set the ClockMark to run or disable it</param>
		private void UpdateAutoRun(AutoStart asAt, string strApplicationPath, string strProgramName, bool AdToAutoRun)
		{
			try
			{
				RegistryKey rkRun;

				switch (asAt)
				{
					case AutoStart.USER_LOGIN:
						rkRun = Registry.CurrentUser.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", true);
						break;
					case AutoStart.WINDOWS_START:
						rkRun = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", true);
						break;
					default:
						rkRun = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", true);
						break;
				}

				if (AdToAutoRun)
				{
					if (!IsAutoRunSet(asAt))
						rkRun.SetValue(strProgramName, strApplicationPath);
				}
				else
				{
					if (IsAutoRunSet(asAt))
						rkRun.DeleteValue(strProgramName, false);
				}

				rkRun.Close();
			}
			catch (Exception caught)
			{
				this.InformTheUser(
					InformTheUserType.ERROR,
					InformTheUserLocation.GENERAL_OPTIONS,
					"There has been an error during accessing the registry. " + "\r\n" +
					caught.Message + "\r\n\r\n" + "Do you have administration privileges?");

				return;
			}
		}

		/// <summary>
		/// Check if shortcut already exists
		/// </summary>
		/// <param name="sc">Place where to check the shortcut existence</param>
		/// <param name="strShortcutName">Name of the shortcut</param>
		private bool IsShortcutSet(Shortcut sc, string strShortcutName)
		{
			string strDirectoryPath;

			switch (sc)
			{
				case Shortcut.DESKTOP:
					strDirectoryPath = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
					break;
				case Shortcut.QUICK_LAUNCH:
					strDirectoryPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\Microsoft\Internet Explorer\Quick Launch";
					break;
				default:
					strDirectoryPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\Microsoft\Internet Explorer\Quick Launch";
					break;
			}

            return (System.IO.File.Exists(strDirectoryPath + "\\" + strShortcutName + ".lnk"));
		}

		/// <summary>
		/// Update the specified folder by creating or deleting a ClockMark Link
		/// </summary>
		/// <param name="DirectoryPath">The full path of the directory in which the link will reside</param>
		/// <param name="TargetPathName">The path name of the target file for the link</param>
		/// <param name="LinkPathName">The file name for the link itself or, if a path name the directory information will be ignored.</param>
		/// <param name="Create">If true, create the link, otherwise delete it</param>
		private void UpdateShortcut(Shortcut sc, string strTargetPathName, string strShortcutName, bool CreateShortcut)
		{
			string strDirectoryPath;

			switch (sc)
			{
				case Shortcut.DESKTOP:
					strDirectoryPath = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
					break;
				case Shortcut.QUICK_LAUNCH:
					strDirectoryPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\Microsoft\Internet Explorer\Quick Launch";
					break;
				default:
					strDirectoryPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\Microsoft\Internet Explorer\Quick Launch";
					break;
			}

			// create the shortcut file itself out of nothing
			DirectoryInfo diSpecialDir = new DirectoryInfo(strDirectoryPath);
			FileInfo fiOriginalFile = new FileInfo(strShortcutName);
			string strNewFileName = diSpecialDir.FullName + "\\" + fiOriginalFile.Name + ".lnk";
			FileInfo fiShortcutFile = new FileInfo(strNewFileName);

			if (CreateShortcut)
			{
				if (!fiShortcutFile.Exists)
				{
					try
					{
						WshShell shell = new WshShell();
						IWshShortcut shortcut = (IWshShortcut)shell.CreateShortcut(fiShortcutFile.FullName);
						shortcut.TargetPath = strTargetPathName;
						shortcut.IconLocation = strTargetPathName;
						shortcut.Save();
					}
					catch
					{
						this.InformTheUser(
							InformTheUserType.ERROR,
							InformTheUserLocation.GENERAL_OPTIONS,
							"Shell Link Error: Unable to create shortcut in special directory: " + strNewFileName);
					}
				}
			}
			else
			{
				if (fiShortcutFile.Exists)
				{
					try
					{
						fiShortcutFile.Delete();
					}
					catch
					{
						this.InformTheUser(
							InformTheUserType.ERROR,
							InformTheUserLocation.GENERAL_OPTIONS,
							"Shell Link Error: Unable to delete shortcut link in special directory: " + strNewFileName);
					}
				}
			}
		}

        private void PingServerAsync(string server)
        {
            Ping pingServer = new Ping();

            pingServer.PingCompleted +=
                new PingCompletedEventHandler(pingServer_PingCompleted);

            pingServer.SendAsync(this.tbServer.Text, null);
        }

        private void pingServer_PingCompleted(object sender, PingCompletedEventArgs e)
        {
            // Check to see if an error occurred.  If no error, then display 
            // the address used and the ping time in milliseconds.
            if (e.Error == null)
            {
                if (e.Cancelled)
                {
                    this.InformTheUser(
                        InformTheUserType.INFO,
                        InformTheUserLocation.SERVER_OPTIONS,
                        "Ping cancelled.");
                }
                else
                {
                    if (e.Reply.Status == IPStatus.Success)
                    {
                        this.InformTheUser(
                            InformTheUserType.INFO,
                            InformTheUserLocation.SERVER_OPTIONS,
                            e.Reply.Address.ToString() + " " +
                            e.Reply.RoundtripTime.ToString(NumberFormatInfo.CurrentInfo) +
                            "ms");
                    }
                    else
                    {
                        this.InformTheUser(
                            InformTheUserType.WARNING,
                            InformTheUserLocation.SERVER_OPTIONS,
                            GetStatusString(e.Reply.Status));
                    }
                }
            }
            else
            {
                // Otherwise display the error.
                this.InformTheUser(
                    InformTheUserType.WARNING,
                    InformTheUserLocation.SERVER_OPTIONS,
                    "Ping error: " +
                    e.Error.InnerException.Message);
            }

            this.btCheckServer.Enabled = true;
        }

		private string GetStatusString(IPStatus status)
		{
			switch (status)
			{
				case IPStatus.Success:
					return "Success.";
				case IPStatus.DestinationHostUnreachable:
					return "Destination host unreachable.";
				case IPStatus.DestinationNetworkUnreachable:
					return "Destination network unreachable.";
				case IPStatus.DestinationPortUnreachable:
					return "Destination port unreachable.";
				case IPStatus.DestinationProtocolUnreachable:
					return "Destination protocol unreachable.";
				case IPStatus.PacketTooBig:
					return "Packet too big.";
				case IPStatus.TtlExpired:
					return "TTL expired.";
				case IPStatus.ParameterProblem:
					return "Parameter problem.";
				case IPStatus.SourceQuench:
					return "Source quench.";
				case IPStatus.TimedOut:
					return "Timed out.";
				default:
					return "Ping failed.";
			}
		}


        private bool ConnectToTheServer()
        {
            try
            {
                this.frmParent.UUID[0] = this.tbUsername.Text.Trim();
                this.frmParent.UUID[1] = Properties.Settings.Default.HashedPassword;
                this.frmParent.UUID[2] = Dns.GetHostName();
                this.frmParent.UUID[3] = this.frmParent.GetIP();

                ClockMarkServiceClient client = new ClockMarkServiceClient(
                    "WSHttpBinding_IClockMarkService",
                    "http://" + this.tbServer.Text.Trim() + ":5555/ClockMarkService/");

                if (client != null)
                {
                    if (!client.IsExistingUser(this.frmParent.UUID))
                    {
                        DialogResult dr = MessageBox.Show("This user does not exist. Do you want to create it?\n(or you just misspell it and you want to enter it again)",
                            "Create new user account?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);

                        if (dr == DialogResult.Yes)
                        {
                            // if user is new
                            if (!client.CreateNewUser(this.frmParent.UUID))
                                throw new Exception("user \"" + this.frmParent.UUID[0] + "\" could not be created on the server");

                            if (!client.IsUserLoginValid(this.frmParent.UUID))
                                throw new Exception("user \"" + this.frmParent.UUID[0] + "\" was not created properly on the server");
                        }
                        else
                            return false;
                    }
                    else
                    {
                        // if user already exists
                        if (!client.IsUserLoginValid(this.frmParent.UUID))
                            throw new Exception("user \"" + this.frmParent.UUID[0] + "\" password is not correct.");
                    }

                    client.Close();
                    return true;
                }

                return false;
            }
            catch (Exception ex)
            {
                this.InformTheUser(
                    InformTheUserType.ERROR,
                    InformTheUserLocation.SERVER_OPTIONS,
                    ex.Message);
            }

            return false;
        }

		#endregion
    }
}
